# -*- coding: utf-8 -*-

import datetime
import time

import pytz
import dateutil
import dateutil.parser

from amazon_reports_downloader import (
    logger,
    get_shared_driver,
    MARKETPLACE_MAPPING,
    DEBUG,
)
from amazon_reports_downloader.helpers import SellerLoginHelper
from amazon_reports_downloader.operation_recorder import OperationRecorder
from amazon_reports_downloader.sku_manager import SkuManager
from amazon_reports_downloader.transfer_manager import TransferManager
from amazon_reports_downloader.disbursements_manager import DisbursementsManager
from amazon_reports_downloader.exchange_rate import ExchangeRate

from amazon_reports_downloader.utils import close_web_driver
from amazon_reports_downloader.lib.google.sheet_api import SheetAPI
from amazon_reports_downloader.lib.telegram_bot import TelegramBot
from amazon_reports_downloader.lib.misc import get_chrome_path
from amazon_reports_downloader.lib.utils.puppeteer_utils import PuppeteerUtils
from pyppeteer.errors import TimeoutError


chicagoTz = pytz.timezone("America/Chicago")


class SkuDeleter:
    def __init__(
        self,
        seller_id,
        marketplace,
        email,
        password,
        sku,
        code=None,
        service_account_path=None,
    ):
        self.sku = sku
        self.code = code
        self.seller_id = seller_id
        self.marketplace = marketplace
        self.email = email
        self.password = password
        self.operation_recorder = OperationRecorder()
        self.disburse_date_format = "%Y-%m-%d"
        self.file_upload_url_template = "https://{}/listing/upload"
        self.disbursements_url_template = (
            "https://{}/payments/allstatements/index.html?ref_=xx_disbmt_ttab_dash"
        )
        self.transfered_alerts = [
            "You cannot disburse more than once in a twenty-four hour period",
            "You cannot disburse if your available balance is zero or less",
            "You can only disburse if your available balance is above the minimum",
        ]
        self.stop_statuses = [
            "REQUEST_TRANSFER_UNAVAILABLE",
            "REQUEST_TRANSFER_DISABLED",
            "INSUFFICIENT_INSTANT_TRANSFER_BALANCE",
            "ALREADY_DISBURSED",
            "TRANSFER_UNAVAILABLE",
        ]
        self.save_statuses = [
            "REQUEST_TRANSFER_UNAVAILABLE",
            "REQUEST_TRANSFER_DISABLED",
            "INSUFFICIENT_INSTANT_TRANSFER_BALANCE",
            "ALREADY_DISBURSED",
            "TRANSFER_UNAVAILABLE",
            "TRANSFER_SUCCEED",
        ]
        self.ignore_statuses = ["REQUEST_TRANSFER_FAILURE", "TRANSFER_FAILED"]
        self.bot = TelegramBot(
            api_token="6140656516:AAFRUeY4muuiIQ8uYndAB_EYUnnMescLWKU"
        )
        self.group_chat_id = "-1001530757239"
        self.pu = PuppeteerUtils(marketplace)
        self.sheet_api = SheetAPI(service_account_path)

    async def run(self):
        self.min_disburse_amount = 200
        self.sheet_id = ""
        page = None
        try:
            try:
                page = await self.get_dashboard(
                    self.seller_id,
                    self.marketplace,
                    self.email,
                    self.password,
                    self.code,
                )
                helper = SellerLoginHelper(
                    page,
                    self.email,
                    self.password,
                    self.marketplace,
                    self.code,
                    self.sheet_api,
                    self.sheet_id,
                )
                if await helper.is_login_required():
                    raise Exception("Login Required!!!")
            except TimeoutError:
                pass
            except Exception as e:
                logger.exception(e)

                record = {
                    "status": "LOGIN_REQUIRED",
                    "seller_id": self.seller_id,
                    "account_code": self.code,
                    "marketplace": self.marketplace,
                    "total_balance": "",
                    "unavailable_balance": "",
                    "instant_transfer_balance": "",
                    "request_transfer_available": "",
                    "message": "Login Required!!!",
                }

                if str(e).find("Login Required") != -1:
                    message = "[LoginRequired] SellerID: {}, Marketplace: {}, Code: {}\n".format(
                        self.seller_id, self.marketplace, self.code
                    )
                else:
                    message = "[Error] Could not access sellercentral, bug fix is needed! SellerID: {}, Marketplace: {}, Code: {}\n".format(
                        self.seller_id, self.marketplace, self.code
                    )
                try:
                    self.bot.send_message(self.group_chat_id, message)
                except:
                    pass

                self.pu.browser().close()
                return

            if page is None:
                return

            try:
                await self.delete_sku(
                    self.seller_id,
                    self.marketplace,
                    self.email,
                    self.password,
                )

                return

            except Exception as e:
                logger.exception(e)
            finally:
                try:
                    self.pu.browser().close()
                except Exception as e:
                    pass
        finally:
            try:
                self.pu.browser().close()
            except Exception as e:
                pass

    async def delete_sku(
        self,
        seller_id,
        marketplace,
        email,
        password,
    ):
        sc_host = MARKETPLACE_MAPPING.get(marketplace.lower())["sellercentral"]
        sku_search_url = f'https://{sc_host}/inventory?tbla_myitable=sort:{"sortOrder":"DESCENDING","sortedColumnId":"date"};search:{sku};pagination:1;'
        page = await self.pu.fetch_url_v2(sku_search_url)
        if not page:
            return

        sku_manager = SkuManager(page, marketplace)
        retries = 7

        selector = ".a-popover.a-splitdropdown.a-dropdown-common.a-declarative"
        while retries > 0:
            try:
                triggered = await sku_manager.trigger_action()
                if not triggered:
                    continue

                visible = await sku_manager.is_element_visible(selector)
                if not visible:
                    continue

            except Exception as e:
                logger.exception(e)

            retries -= 1

            await page.reload()
            time.sleep(3)

    async def get_dashboard(
        self,
        seller_id,
        marketplace,
        email,
        password,
        code=None,
    ):
        sc_host = MARKETPLACE_MAPPING.get(marketplace.lower())["sellercentral"]

        seller_central_url = "https://{}/home".format(sc_host)

        page = await self.pu.fetch_url_v2(seller_central_url)
        helper = SellerLoginHelper(
            page, email, password, marketplace, code, self.sheet_api
        )

        is_logged = await helper.is_login_required()
        while is_logged:
            logger.info("Login required! Trying to login...")

            await helper.login()
            await helper.send_otp()
            await helper.check_opt()
            await helper.select_country()

            wait_time = 180
            while wait_time > 0:
                wait_time -= 1
                logger.debug("Waiting for login...")
                if await helper.is_login_required():
                    time.sleep(2)
                else:
                    break

            if wait_time <= 0:
                message = (
                    "[LoginSellerCentralFailed] SellerID: {}, Marketplace: {}".format(
                        seller_id, marketplace
                    )
                )
                logger.error(message)
                return page

            time.sleep(7)

            is_logged = await helper.is_login_required()

        await helper.select_country()
        if await helper.is_login_required():
            message = "[LoginSellerCentralFailed] SellerID: {}, Marketplace: {}".format(
                seller_id, marketplace
            )
            logger.error(message)
            return page

        logger.info("begin to pick marketplace")
        await helper.pick_marketplace()
        logger.info("Picked marketplace!")

        return page
